<?php

/**
 * Classe per interagire con l'API REST di gCloud Storage
 *
 * @version 1.1.59
 * @package GCloud-SDK-PHP
 */
/**
 * Dipendiamo dalla HTTP_Request2 di PEAR
 */
require_once 'HTTP/Request2.php';

/**
 * Definizione della classe principale
 * 
 * @author Andrea Brancatelli <abrancatelli@schema31.it>
 * @package gCloud_Storage
 */
class gCloud_Storage {

    /**
     * Attuale versione della libreria
     *
     * @var string
     * @access public
     */
    const VERSION = "gCloud_Storage 1.1.9 [Pear]";

    /**
     * Mime-type standard da utilizzare quando non specificato
     */
    const DEFAULT_MIME_TYPE = "application/octet-stream";

    /**
     * Repository su cui operare
     *
     * @var string 
     * @access public
     */
    public $repositoryName = "";

    /**
     * Chiave di autenticazione
     *
     * @var string 
     * @access public
     */
    public $Authentication = "";

    /**
     * gCloud Storage Base Path (hardcoded a "storage.gcloud.schema31.it)
     *
     * @var string 
     * @access public
     */
    public $gCloudStorageHost = "storage.gcloud.schema31.it";

    /**
     * L'ultimo errore ricevuto
     *
     * @var string 
     * @access public
     */
    public $LastError = "";

    /**
     * Variabili di appoggio contenenti la risposta del server
     */
    public $ResponseBody = "";
    public $ResponseStatus = "";

    /**
     * Numero di tentativi per cui riprovare ad eseguire l'operazione richiesta in caso di errori
     *
     * @var int 
     * @access public
     */
    public $maxAutoRetry = 1;

    /**
     * Contatore delle iterazioni (maxAutoRetry) eseguite.
     * Default 1
     *
     * @var int 
     * @access private
     */
    private $lastAutoRetry = 0;

    /**
     * Tempo di attesa tra un tentativo e l'altro in caso di 'autoRetry' (espesso in microsecondi).
     * default 1 secondo
     *
     * @var int 
     * @access public
     */
    public $autoRetrySleep = 1000000;

    /**
     * Contenitore per i dettagli di un file.
     * Viene inizializzato dal metodo 'getFile' per fare in modo che, a fronte di un download, 
     * sia possibile accedere anche ai metadati del file, oltre che al suo contenuto.
     *
     * @var stdClass
     * @access public
     */
    public $fileDetails = NULL;

    /**
     * script timeout (espesso in secondi)
     *
     * @var int 
     * @access public
     */
    const SCRIPT_TIMEOUT = 300;

    /**
     * Mappatura codici di ritorno
     */
    //private $putStatusCodeOK = array(200);
    private $postStatusCodeOK = array(201);
    private $getStatusCodeOK = array(200);
    private $deleteStatusCodeOK = array(200);
    private $patchStatusCodeOK = array(204);

    /**
     * Costruttore della classe gCloud_Storage
     */
    function __construct() {

        /*
         *  se la configurazione prevede un 'default_socket_timeout' minore di 'SCRIPT_TIMEOUT', usa 'SCRIPT_TIMEOUT'
         */
        if (ini_get('default_socket_timeout') < self::SCRIPT_TIMEOUT) {
            ini_set("default_socket_timeout", self::SCRIPT_TIMEOUT);
        }

        /*
         *  se la configurazione prevede un 'max_execution_time' minore di 'SCRIPT_TIMEOUT', usa 'SCRIPT_TIMEOUT'
         */
        if (ini_get('max_execution_time') < self::SCRIPT_TIMEOUT) {
            ini_set("max_execution_time", self::SCRIPT_TIMEOUT);
        }
    }

    /**
     * Wrapper generale per tutti i metodi esposti dalla classe.
     * Gestisce le iterazioni 'autoRetry' ed i relativi 'autoRetrySleep' in base al valore ritornato ($response) dal metodo richiesto.
     * Se $response é FALSE, viene eseguito nuovamente il metodo richiesto, fino al raggiungimento del numero massimo di iterazioni 
     * eseguibili, aspettando 'autoRetrySleep' tra un'iterazione e l'altra
     * @return mixed output del metodo chiamato
     * @access public
     */
    public function __call($method, $arguments) {

        $response = FALSE;

        /*
         *  gestione iterazioni
         */
        while ($response === FALSE && $this->lastAutoRetry <= (int) $this->maxAutoRetry) {

            try {

                /*
                 * esegue il metodo richiesto
                 */
                $response = call_user_func_array(array($this, $method), $arguments);

                if ($response === FALSE) {

                    /*
                     *  esito negativo - prova ad eseguire nuovamente l'operazione richiesta
                     */
                    $this->lastAutoRetry++;
                    usleep((int) $this->autoRetrySleep);
                } else {
                    /*
                     *  OK - esce dal ciclo e ritorna l'esito
                     */
                    break;
                }
            } catch (Exception $e) {

                /*
                 *  catch eccezione - prova ad eseguire nuovamente l'operazione richiesta
                 */
                $response = FALSE;
                $this->LastError = 'Caught exception: ' . $e->getMessage();

                $this->lastAutoRetry++;
                usleep((int) $this->autoRetrySleep);
            }
        }

        return $response;
    }

    /**
     * Torna le informazioni su un dato repository
     * @return mixed torna i dettagli sul repository richiesto oppure FALSE in caso di errore
     * @access protected
     */
    protected function detailRepository() {

        /**
         * Preparo il base url
         */
        $url = "http://{$this->gCloudStorageHost}/api/repository";

        /*
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $request = new HTTP_Request2($url);
        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod(HTTP_Request2::METHOD_GET)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION);

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->getStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /**
         * Gestiamo il contenuto della risposta
         */
        $response = json_decode($this->ResponseBody);
        if (!is_object($response) || property_exists($response, 'Error')) {
            $this->LastError = $this->ResponseBody;
            return FALSE;
        }

        /*
         * Ritorna i dettagli sul file richesto
         */
        return $response;
    }

    /**
     * Ritorna l'elenco dei file presenti nel repository selezionato
     * @param int $offset indice da cui iniziare l'estrazione dei dati
     * @param int $limit numero di record da estrarre
     * @param string $sortBy tipologia ordinamento (fileKeyAsc | fileKeyDesc | dateAsc | dateDesc | fileSizeAsc | fileSizeDesc | fileNameAsc | fileNameDesc)
     * @param boolean $getVersions gestione file versionati (TRUE ritorna anche le vecchie versioni per la stessa fileKey, FALSE ritorna solo la versione corrente per la stessa fileKey)
     * @param boolean $isDeleted gestione file cancellati (TRUE ritorna i file cancellati, FALSE ritorna i file non cancellati)
     * @param string $fileName filtra per fileName / fileKey
     * @return boolean|array torna un array di oggetti di tipo stdClass
     * @access protected
     */
    protected function getResourcesList($offset = NULL, $limit = NULL, $sortBy = NULL, $getVersions = NULL, $isDeleted = NULL, $fileName = NULL) {

        /**
         * Preparo il base url
         */
        $url = "http://{$this->gCloudStorageHost}/api/resources";

        /*
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */

        $request = new HTTP_Request2($url);
        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod(HTTP_Request2::METHOD_GET)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION)
                ->setHeader('offset', (strlen($offset) > 0 ? $offset : 0))
                ->setHeader('limit', (strlen($limit) > 0 ? $limit : 50))
                ->setHeader('sortBy', (strlen($sortBy) > 0 ? $sortBy : 'dateAsc'))
                ->setHeader('getVersions', ($getVersions == TRUE ? 1 : ""))
                ->setHeader('isDeleted', ($isDeleted == TRUE ? 1 : ""))
                ->setHeader('fileName', (strlen($fileName) > 0 ? $fileName : ""));

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->getStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /*
         * Ritorna l'elenco dei file
         */
        return json_decode($this->ResponseBody);
    }

    /**
     * Carica un documento su gCloud Storage
     *
     * @param string $localFile Path del file da inviare su questo filesystem
     * @param string $mime Mime part del file da inviare (default application/octet-stream)
     * @param string $publicName Nome del file da inviare a gCloud Storage (di default prende il basename di $localFile)
     * @param string $fileKey Riferimento ad una fileKey già esistente per effettuare la "sovrascrittura" di un file
     * @return boolean|string torna la fileKey ritornata da gCloud Storage oppure FALSE in caso di errore
     * @access protected
     */
    protected function sendFile($localFile = NULL, $mime = self::DEFAULT_MIME_TYPE, $publicName = NULL, $fileKey = NULL) {

        /**
         * Verifico se il file esiste
         */
        if (strlen($localFile) == 0 || !file_exists($localFile)) {
            $this->LastError = "File not found: {$localFile}";
            return FALSE;
        }

        /**
         * Se non ci viene passato un publicName esplicito lo ricaviamo dal nome del file da inviare
         */
        $publicName = (strlen($publicName) > 0 ? $publicName : basename($localFile));

        /**
         * Preparo il base url
         */
        $url = "http://{$this->gCloudStorageHost}/api/resource/";

        /**
         * E' stata indicata una fileKey?
         */
        $url .= (strlen($fileKey) > 0 ? "fileKey/{$fileKey}" : "");

        /**
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $request = new HTTP_Request2($url);
        $request->setMethod(HTTP_Request2::METHOD_POST)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION)
                ->setHeader('Expect', "")
                ->addUpload('somefile', $localFile, $publicName, $mime);

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->postStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /**
         * Gestiamo il contenuto della risposta
         */
        $response = json_decode($this->ResponseBody);
        if (!is_object($response) || property_exists($response, 'Error')) {
            $this->LastError = $this->ResponseBody;
            return FALSE;
        }

        /**
         * Ok - ritorno la fileKey 
         */
        return $response->Result[0]->fileKey;
    }

    /**
     * Carica N documenti su gCloud Storage
     * 
     * @param array $localFiles Array contenente i file per cui effettuare l'upload
     * @param string $fileKey Riferimento ad una fileKey già esistente per effettuare la "sovrascrittura" di un file
     * @return boolean|array torna le fileKey ritornate da gCloud Storage oppure FALSE in caso di errore
     * @access protected
     */
    protected function sendMultipleFiles($localFiles = array(), $fileKey = NULL) {

        /**
         * Preparo il base url
         */
        $url = "http://{$this->gCloudStorageHost}/api/resource/";

        /**
         * E' stata indicata una fileKey?
         */
        $url .= (strlen($fileKey) > 0 ? "fileKey/{$fileKey}" : "");

        /*
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $request = new HTTP_Request2($url);
        $request->setMethod(HTTP_Request2::METHOD_POST)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION)
                ->setHeader('Expect', "");

        /**
         * Fetch dell'array contenente i riferimenti ai file da inviare
         */
        foreach ($localFiles as $index => $localFile) {

            /*
             * estrae il file path
             */
            if (!array_key_exists('path', $localFile) || !file_exists($localFile['path'])) {
                $this->LastError = "Malformed file list";
                return FALSE;
            }
            $localFilePath = $localFile['path'];

            /*
             * estrae il mime-type
             */
            $mime = (array_key_exists('mime', $localFile) && strlen($localFile['mime']) > 0 ? $localFile['mime'] : self::DEFAULT_MIME_TYPE);

            /*
             * Se non ci viene passato un publicName esplicito lo ricaviamo dal nome del file da inviare
             */
            $publicName = ( array_key_exists('publicName', $localFile) && strlen($localFile['publicName']) > 0 ? $localFile['publicName'] : basename($localFilePath) );

            /*
             *  Aggiunge all'oggetto HTTP_Request2 il file
             */
            $request->addUpload("fileUpload_{$index}", $localFilePath, $publicName, $mime);
        }

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->postStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /**
         * Gestiamo il contenuto della risposta
         */
        $response = json_decode($this->ResponseBody);
        if (!is_object($response) || property_exists($response, 'Error')) {
            $this->LastError = $this->ResponseBody;
            return FALSE;
        }

        /**
         * Ok - ritorno le fileKey 
         */
        return $response->Result;
    }

    /**
     * Torna le informazioni su un dato file
     * @param string $fileKey identificativo del file
     * @param int $fileVersion versione del file
     * @param string $chain nome della chain (di tipo output) da applicare
     * @param int $subFile indice del file prodotto della chain (di tipo output)
     * @return mixed torna i dettagli sulla risorsa richiesta oppure FALSE in caso di errore
     * @access protected
     */
    protected function detailFile($fileKey = NULL, $fileVersion = NULL, $chain = NULL, $subFile = NULL) {

        /*
         * E' stata indicata una fileKey?
         */
        if (strlen($fileKey) == 0) {
            $this->LastError = "No fileKey supplied";
            return FALSE;
        }

        /**
         * Preparo il base url
         */
        $url = "http://{$this->gCloudStorageHost}/api/resource/fileKey/{$fileKey}";

        /*
         * se non è stata indicata una versione specifica o è stato passato il carattere '_', usa l'ultima versione disponibile
         */
        $url .= (strlen($fileVersion) > 0 && $fileVersion != '_' ? "/fileVersion/$fileVersion" : '/_' );

        /*
         *  se si sta richedendo un file generato da una chain, è necessario indicare sia il nome della chain che l'indice del file di output
         */
        if ((strlen($chain) > 0 && strlen($subFile) == 0) || (strlen($chain) == 0 && strlen($subFile) > 0)) {
            $this->LastError = "No chain-name / chain-fileIndex supplied";
            return FALSE;
        } elseif (strlen($chain) > 0 && strlen($subFile) > 0) {
            $url .= "/chain/$chain/chainFileIndex/$subFile";
        }

        /*
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $request = new HTTP_Request2($url);
        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod(HTTP_Request2::METHOD_GET)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION);

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->getStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /**
         * Gestiamo il contenuto della risposta
         */
        $response = json_decode($this->ResponseBody);
        if (!is_object($response) || property_exists($response, 'Error')) {
            $this->LastError = $this->ResponseBody;
            return FALSE;
        }

        /*
         * Ritorna i dettagli sul file richesto
         */
        return $response;
    }

    /**
     * Scarica un documento da gCloud Storage
     * @param string $fileKey identificativo del file
     * @param int $fileVersion versione del file
     * @param string $chain nome della chain (di tipo output) da applicare
     * @param int $subFile indice del file prodotto della chain (di tipo output)
     * @param array $pluginParameters parametri di configurazione aggiuntivi
     * @return mixed torna il contenuto della risorsa richiesta oppure FALSE in caso di errore
     * @access protected
     */
    protected function getFile($fileKey = NULL, $fileVersion = NULL, $chain = NULL, $subFile = NULL, $pluginParameters = array()) {

        /*
         * E' stata indicata una fileKey?
         */
        if (strlen($fileKey) == 0) {
            $this->LastError = "No fileKey supplied";
            return FALSE;
        }

        /*
         *  se si sta richedendo un file generato da una chain, è necessario indicare sia il nome della chain che l'indice del file di output
         */
        if ((strlen($chain) > 0 && strlen($subFile) == 0) || (strlen($chain) == 0 && strlen($subFile) > 0)) {
            $this->LastError = "No chain-name / chain-fileIndex supplied";
            return FALSE;
        }
        /*
         * se non è stata indicata una versione specifica o è stato passato il carattere '_', usa l'ultima versione disponibile
         */
        $fileVersion = (strlen($fileVersion) > 0 && $fileVersion != '_' ? $fileVersion : '_' );


        /*
         * anche in presenza degli attributi $chain e $subFile, vengono estratte le informazioni relative alla risorsa 'master' in quanto:
         * 1. la risorsa 'master' è utilizzata come riferimento anche per tutte le eventuali risorse generate da una chain
         * 2. la risorsa a cui si sta puntando potrebbe essere generata da una chain di tipo 'OUTPUT', quindi fisicamente non presente a sistema
         */
        $response = $this->detailFile($fileKey, $fileVersion);

        if ($response === FALSE) {
            /*
             *  file non valido - l'attributo 'LastError' è stato settato nel metodo 'detailFile'
             */
            return FALSE;
        }

        /*
         *  rendo accessibile i dettagli relativi alla risorsa 'master'
         */
        $this->fileDetails = $response;

        /**
         * Preparo il base url
         */
        $url = "$response->friendlyUrl";

        /*
         * Se la chiamata al metodo 'detailFile' è stata effettuata indicando una fileVersion 
         * specifica (non '_'), la fileVersion sarà già stata inclusa nel friendlyUrl, non va 
         * pertanto aggiunta nuovamente all'URL 
         */
        $url .= ($fileVersion == "_" ? "/_" : "");

        /*
         *  aggiungo al base url i riferimenti alla chain (se indicata) e gli eventuali parametri aggiuntivi per le plugin
         */
        if (strlen($chain) > 0 && strlen($subFile) > 0) {

            /*
             * Aggiorno il base url
             */
            $url .= "/$chain/$subFile";

            /*
             * Gestisco i parametri di configurazione aggiuntivi
             */
            foreach ($pluginParameters as $pluginName => $parameters) {

                $url .= "/{$pluginName}:";

                foreach ($parameters as $parameter) {

                    if (is_string($parameter) || is_numeric($parameter)) {
                        $url .= "{$parameter}~";
                    }
                }

                /*
                 *  Rimuoviamo l'ultimo '~' (se presente)
                 */
                $url = rtrim($url, '~');
            }
        }

        /*
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $request = new HTTP_Request2($url);
        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod(HTTP_Request2::METHOD_GET)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION);

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->getStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /*
         * Ritorna il contenuto del file richiesto
         */
        return $this->ResponseBody;
    }

    /**
     * Ripristina un file precedentemente cancellato su gCloud Storage
     * @param string $fileKey identificativo del file
     * @param int $fileVersion versione del file
     * @return mixed torna i dettagli sulla risorsa modificata oppure FALSE in caso di errore
     * @access protected
     */
    protected function undeleteFile($fileKey = NULL, $fileVersion = NULL) {

        /*
         * E' stata indicata una fileKey?
         */
        if (strlen($fileKey) == 0) {
            $this->LastError = "No fileKey supplied";
            return FALSE;
        }

        /**
         * Preparo il base url
         */
        $url = "http://{$this->gCloudStorageHost}/api/resource/fileKey/{$fileKey}";

        /*
         * Gestisce l'eventuale parametro $fileVersion
         */
        $url .= (strlen($fileVersion) > 0 ? "/fileVersion/{$fileVersion}" : "");

        $request = new HTTP_Request2($url);
        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod('PATCH')
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('undeleteFile', 1)
                ->setHeader('User-Agent', self::VERSION);

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->patchStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /**
         * Gestiamo il contenuto della risposta
         */
        $response = json_decode($this->ResponseBody);
        if (!is_object($response) || property_exists($response, 'Error')) {
            $this->LastError = $this->ResponseBody;
            return FALSE;
        }

        /*
         * Ritorna la risposta
         */
        return $response;
    }

    /**
     * Modifica un file su gCloud Storage
     * @param string $fileKey identificativo del file
     * @param int $fileVersion versione del file
     * @param string $fileName il nuovo nome da associare al file
     * @param string $fileMimeType il nuovo mime-type da associare al file
     * @return mixed torna i dettagli sulla risorsa modificata oppure FALSE in caso di errore
     * @access protected
     */
    protected function editFile($fileKey = NULL, $fileVersion = NULL, $fileName = NULL, $fileMimeType = NULL) {

        /*
         * E' stata indicata una fileKey?
         */
        if (strlen($fileKey) == 0) {
            $this->LastError = "No fileKey supplied";
            return FALSE;
        }

        /**
         * Preparo il base url
         */
        $url = "http://{$this->gCloudStorageHost}/api/resource/fileKey/{$fileKey}";

        /*
         * Gestisce l'eventuale parametro $fileVersion
         */
        $url .= (strlen($fileVersion) > 0 ? "/fileVersion/{$fileVersion}" : "");

        $request = new HTTP_Request2($url);
        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod('PATCH')
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION);

        // rinominare il file?
        if (strlen($fileName) > 0) {
            $request
                    ->setHeader('editFileName', 1)
                    ->setHeader('fileName', $fileName);
        }

        // rettificare il mime type?
        if (strlen($fileMimeType) > 0) {
            $request
                    ->setHeader('editFileMimeType', 1)
                    ->setHeader('fileMimeType', $fileMimeType);
        }

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->patchStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /**
         * Gestiamo il contenuto della risposta
         */
        $response = json_decode($this->ResponseBody);
        if (!is_object($response) || property_exists($response, 'Error')) {
            $this->LastError = $this->ResponseBody;
            return FALSE;
        }

        /*
         * Ritorna la risposta
         */
        return $response;
    }

    /**
     * Cancella un file da gCloud Storage
     * @param string $fileKey identificativo del file
     * @param int $fileVersion versione del file
     * @return mixed torna i dettagli sulla risorsa cancellata oppure FALSE in caso di errore
     * @access protected
     */
    protected function deleteFile($fileKey = NULL, $fileVersion = NULL) {

        /*
         * E' stata indicata una fileKey?
         */
        if (strlen($fileKey) == 0) {
            $this->LastError = "No fileKey supplied";
            return FALSE;
        }

        /**
         * Preparo il base url
         */
        $url = "http://{$this->gCloudStorageHost}/api/resource/fileKey/{$fileKey}";

        /*
         * Gestisce l'eventuale parametro $fileVersion
         */
        $url .= (strlen($fileVersion) > 0 ? "/fileVersion/{$fileVersion}" : "");

        $request = new HTTP_Request2($url);
        $request->setConfig('follow_redirects', TRUE);
        $request->setConfig('max_redirects', 20);
        $request->setConfig('strict_redirects', TRUE);
        $request->setMethod(HTTP_Request2::METHOD_DELETE)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('User-Agent', self::VERSION);

        /**
         * Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->deleteStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /**
         * Gestiamo il contenuto della risposta
         */
        $response = json_decode($this->ResponseBody);
        if (!is_object($response) || property_exists($response, 'Error')) {
            $this->LastError = $this->ResponseBody;
            return FALSE;
        }

        /*
         * Ritorna la risposta
         */
        return $response;
    }

    /**
     * Ritorna lo short-url per accedere ad una risorsa memorizzata all'interno di un repository privato
     * @param string $fileKey
     * @param int $fileVersion (facoltativo - se omesso ritorna l'ultima versione per la fileKey passata come parametro)
     * @param int $expiresAt (facoltativo - indica il numero di secondi per cui il link generato sarà valido)
     * @param string $whitelist (facoltativo - lista di indirizzi IP, separati da punto e virgola, abilitati ad accedere al link generato )
     * @return boolean|string url della risorsa oppure FALSE in caso di errore
     * @access protected
     */
    protected function getShortUrl($fileKey = NULL, $fileVersion = NULL, $expiresAt = NULL, $whitelist = NULL) {

        /*
         * E' stata indicata una fileKey?
         */
        if (strlen($fileKey) == 0) {
            $this->LastError = "No fileKey supplied";
            return FALSE;
        }

        /**
         * Preparo il base url
         */
        $url = "http://{$this->gCloudStorageHost}/api/shorturl";

        /*
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $request = new HTTP_Request2($url);
        $request->setMethod(HTTP_Request2::METHOD_POST)
                ->setHeader('Authentication', $this->Authentication)
                ->setHeader('repositoryName', $this->repositoryName)
                ->setHeader('fileKey', $fileKey)
                ->setHeader('User-Agent', self::VERSION)
                ->setHeader('Expect', "");

        /*
         *  è stata indicata una fileVersion?
         */
        if (strlen($fileVersion) > 0) {
            $request->setHeader('fileVersion', $fileVersion);
        }

        /*
         *  è stata indicata una durata massima per la validità dell'url?
         */
        if (strlen($expiresAt) > 0) {
            $request->setHeader('expiresAt', $expiresAt);
        }

        /*
         *  è stata indicata una white-list di indirizzi IP, i quali potranno accedere alla risorsa?
         */
        if (strlen($whitelist) > 0) {
            $request->setHeader('whitelist', $whitelist);
        }

        /*
         *  Catturo la response della richiesta
         */
        $HTTPRequest2Response = $request->send();
        $this->ResponseBody = $HTTPRequest2Response->getBody();
        $this->ResponseStatus = $HTTPRequest2Response->getStatus();

        /*
         * Validazione http status code
         */
        if (!in_array($this->ResponseStatus, $this->postStatusCodeOK)) {
            $this->LastError = "Response status code: {$this->ResponseStatus}";
            return FALSE;
        }

        /**
         * Gestiamo il contenuto della risposta
         */
        $response = json_decode($this->ResponseBody);
        if (!is_object($response) || property_exists($response, 'Error')) {
            $this->LastError = $this->ResponseBody;
            return FALSE;
        }

        /*
         * ritorna lo short url
         */
        return $response->shortUrl;
    }

}
